/*
 * Copyright (c) 2005-2007 Extreme! Lab, Indiana University. All rights reserved.
 *
 * This software is open source. See the bottom of this file for the license.
 *
 * $Id: GPELClient.java,v 1.52 2007/05/08 18:42:12 sshirasu Exp $
 */
package edu.indiana.extreme.xbaya.gpel;

import java.awt.image.BufferedImage;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.net.URI;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.imageio.ImageIO;

import org.gpel.GpelConstants;
import org.gpel.client.GcBinaryWebResource;
import org.gpel.client.GcInstance;
import org.gpel.client.GcProcessResource;
import org.gpel.client.GcProvidesWsdlResource;
import org.gpel.client.GcResourceNotFoundException;
import org.gpel.client.GcSearchList;
import org.gpel.client.GcSearchRequest;
import org.gpel.client.GcTemplate;
import org.gpel.client.GcUsesWsdlResource;
import org.gpel.client.GcWebResource;
import org.gpel.client.GcWsdlResource;
import org.gpel.client.GcXmlWebResource;
import org.gpel.client.GpelClient;
import org.gpel.client.GpelUserCredentials;
import org.gpel.client.http.apache_http_client.Transport;
import org.gpel.client.security.GpelUserX509Credential;
import org.gpel.model.GpelProcess;

import xsul5.MLogger;
import xsul5.wsdl.WsdlDefinitions;
import edu.indiana.extreme.xbaya.XBayaConstants;
import edu.indiana.extreme.xbaya.XBayaRuntimeException;
import edu.indiana.extreme.xbaya.component.ComponentException;
import edu.indiana.extreme.xbaya.component.ws.WorkflowComponent;
import edu.indiana.extreme.xbaya.event.Event;
import edu.indiana.extreme.xbaya.event.EventProducer;
import edu.indiana.extreme.xbaya.gpel.script.BPELScript;
import edu.indiana.extreme.xbaya.graph.Graph;
import edu.indiana.extreme.xbaya.graph.GraphException;
import edu.indiana.extreme.xbaya.graph.util.GraphUtil;
import edu.indiana.extreme.xbaya.graph.ws.WSGraph;
import edu.indiana.extreme.xbaya.graph.ws.WSGraphFactory;
import edu.indiana.extreme.xbaya.graph.ws.WSNode;
import edu.indiana.extreme.xbaya.gui.ErrorMessages;
import edu.indiana.extreme.xbaya.security.SecurityUtil;
import edu.indiana.extreme.xbaya.security.XBayaSecurity;
import edu.indiana.extreme.xbaya.util.WSDLUtil;
import edu.indiana.extreme.xbaya.wf.Workflow;

/**
 * @author Satoshi Shirasuna
 */
public class GPELClient extends EventProducer {

    /**
     * Either workflow template or workflow instance
     */
    public enum WorkflowType {
        /**
         * Workflow template
         */
        TEMPLATE,

        /**
         * Workflow instance
         */
        INSTANCE
    }

    private static final String PROCESS_WSDL_TYTLE = "process.wsdl";

    private static final String PROCESS_GPEL_TITLE = "process.gpel";

    private static final String PNG_MIME_TYPE = "image/png";

    private static final String GRAPH_MIME_TYPE = "application/x-xbaya+xml";

    private final static MLogger logger = MLogger.getLogger();

    private URI engineURL;

    private GpelClient client;

    private GPELLinksFilter linksFilter = new GPELLinksFilter();

    private GpelUserX509Credential gpelUserX509Credential;

    /**
     * Constructs a GPELClient.
     */
    public GPELClient() {
        // Nothing
    }

    /**
     * Constructs a GPELClient.
     * 
     * @param engineURL
     * @param gpelUserX509Credential
     * @throws GPELException
     */
    public GPELClient(URI engineURL,
            GpelUserX509Credential gpelUserX509Credential) throws GPELException {
        this.engineURL = engineURL;
        this.gpelUserX509Credential = gpelUserX509Credential;
        connect();
    }

    /**
     * @param engineURL
     * @throws GPELException
     */
    public void setEngineURL(URI engineURL) throws GPELException {
        this.engineURL = engineURL;
        connect();
    }

    /**
     * @return The URL of the GPEL Engine.
     */
    public URI getEngineURL() {
        return this.engineURL;
    }

    /**
     * @param gpelUserX509Credential
     * @throws GPELException
     */
    public void setGpelUserX509Credential(
            GpelUserX509Credential gpelUserX509Credential) throws GPELException {
        this.gpelUserX509Credential = gpelUserX509Credential;
        connect();
    }

    /**
     * Generates a BPEL process.
     * 
     * @param workflow
     * @throws GraphException
     */
    public static void createScript(Workflow workflow) throws GraphException {
        // Generate a BPEL process.
        BPELScript script = new BPELScript(workflow);
        script.create();
        workflow.setGpelProcess(script.getGpelProcess());
        workflow.setWorkflowWSDL(script.getWorkflowWSDL().getWsdlDefinitions());
    }

    /**
     * Deploys a workflow to the GPEL Engine.
     * 
     * @param workflow
     * @param redeploy
     * @return The workflow template ID.
     * @throws GraphException
     * @throws GPELException
     */
    public URI createScriptAndDeploy(Workflow workflow, boolean redeploy)
            throws GraphException, GPELException {
        logger.entering(new Object[] { workflow });

        // Generate a BPEL process.
        createScript(workflow);

        return deploy(workflow, redeploy);
    }

    /**
     * @param workflow
     * @param redeploy
     * @return The workflow template ID.
     * @throws GPELException
     */
    public synchronized URI deploy(Workflow workflow, boolean redeploy)
            throws GPELException {
        if (workflow.getGpelProcess() == null) {
            throw new IllegalStateException(
                    "BPEL script has to have been generated.");
        }
        if (!isConnected()) {
            throw new IllegalStateException(
                    "The BPEL Engine has not been configured.");
        }

        GcTemplate template = null;
        List<GcWebResource> links;
        GcProcessResource processResource = null;
        GcXmlWebResource graphResource = null;
        GcBinaryWebResource imageResource = null;
        GcWsdlResource workflowWSDLresource = null;
        Map<String, GcWsdlResource> wsdlResourceMap = new HashMap<String, GcWsdlResource>();

        if (redeploy) {
            URI templateID = workflow.getGPELTemplateID();
            if (templateID != null) {
                try {
                    template = this.client.retrieveTemplate(templateID);
                } catch (GcResourceNotFoundException e) {
                    // The workflow was not found in the engine.
                    logger.caught(e);
                    template = null;
                } catch (RuntimeException e) {
                    throw new GPELException(ErrorMessages.GPEL_ERROR, e);
                }
            }
        }

        String title = workflow.getName();
        if (template == null) {
            // This case is either it is a new deploy or the tempate does not
            // exist in the GPEL Engine.

            template = this.client.createTemplate(title);
            links = new ArrayList<GcWebResource>();
            template.setLinks(links);
        } else {
            // Redeploy

            template.setTitle(title);
            links = template.getLinks();

            // Look for each resource.
            for (GcWebResource link : links) {
                String rel = link.getRel();
                logger.finest("rel: " + rel);
                if (GpelConstants.REL_PROCESS.equals(rel)) {
                    processResource = (GcProcessResource) link;
                } else if (GPELLinksFilter.REL_XWF.equals(rel)) {
                    graphResource = (GcXmlWebResource) link;
                } else if (GPELLinksFilter.REL_IMAGE.equals(rel)) {
                    imageResource = (GcBinaryWebResource) link;
                } else if (GpelConstants.REL_WSDL.equals(rel)) {
                    GcWsdlResource wsdlResouece = (GcWsdlResource) link;
                    String wsdlTitle = wsdlResouece.getTitle();
                    logger.finest("wsdlTitle: " + wsdlTitle);
                    // XXX Can we rely on the title to do the matching?
                    wsdlResourceMap.put(wsdlTitle, wsdlResouece);
                }
            }
        }

        // BPEL process
        GpelProcess process = workflow.getGpelProcess();
        if (processResource == null) {
            processResource = new GcProcessResource(PROCESS_GPEL_TITLE, process);
            links.add(processResource);
        } else {
            processResource.setXmlContent(process.xml());
        }

        // WSDL for the process
        workflowWSDLresource = wsdlResourceMap.remove(PROCESS_WSDL_TYTLE);
        WsdlDefinitions workflowWSDL = workflow.getWorkflowWSDL();
        logger.finest(workflowWSDL.xmlString());
        if (workflowWSDLresource == null) {
            workflowWSDLresource = new GcWsdlResource(PROCESS_WSDL_TYTLE,
                    workflowWSDL);
            links.add(workflowWSDLresource);
        } else {
            workflowWSDLresource.setXmlContent(workflowWSDL.xml());
        }

        Map<String, WsdlDefinitions> wsdlMap = workflow.getWSDLs();
        for (String id : wsdlMap.keySet()) {
            WsdlDefinitions wsdl = wsdlMap.get(id);
            GcWsdlResource wsdlResource = wsdlResourceMap.remove(id);
            if (wsdlResource == null) {
                wsdlResource = new GcWsdlResource(id, wsdl);
                links.add(wsdlResource);
            } else {
                wsdlResource.setXmlContent(wsdl.xml());
            }
        }

        // Remove the rest of unused WSDL from the links.
        for (GcWsdlResource resource : wsdlResourceMap.values()) {
            links.remove(resource);
        }

        // Graph
        Graph graph = workflow.getGraph();
        if (graphResource == null) {
            logger.finest("Creating a new graphResource");
            graphResource = new GcXmlWebResource("process.xgr", graph.toXML(),
                    GRAPH_MIME_TYPE);
            graphResource.setRel(GPELLinksFilter.REL_XWF);
            links.add(graphResource);
        } else {
            logger.finest("Updating the graphResource");
            graphResource.setXmlContent(graph.toXML());
        }

        // Image
        try {
            BufferedImage image = workflow.getImage();
            ByteArrayOutputStream outStream = new ByteArrayOutputStream();
            ImageIO.write(image, XBayaConstants.PNG_FORMAT_NAME, outStream);
            byte[] bytes = outStream.toByteArray();

            if (imageResource == null) {
                imageResource = new GcBinaryWebResource("image.png", bytes,
                        PNG_MIME_TYPE);
                imageResource.setRel(GPELLinksFilter.REL_IMAGE);
                links.add(imageResource);
            } else {
                imageResource.setBinaryContent(bytes);
            }
        } catch (IOException e) {
            // It's OK to not save the image for now.
            logger.caught(e);
        } catch (IllegalArgumentException e) {
            // This happens only from junit tests.
            logger.caught(e);
        }

        try {
            this.client.deployTemplate(template);

            // this needs to be kept and reused
            URI templateID = template.getTemplateId();
            workflow.setGPELTemplateID(templateID);

            logger.exiting();
            return templateID;
        } catch (RuntimeException e) {
            throw new GPELException(ErrorMessages.GPEL_ERROR, e);
        }
    }

    /**
     * Loads a workflow with s specified workflow template ID.
     * 
     * @param templateID
     *            The workflow template ID.
     * @return The workflow loaded
     * @throws GraphException
     * @throws GPELException
     * @throws ComponentException
     */
    public Workflow load(URI templateID) throws GraphException, GPELException,
            ComponentException {
        // Don't delete this method because the portal uses it.
        return load(templateID, WorkflowType.TEMPLATE);
    }

    /**
     * @param id
     * @param workflowType
     * @return The workflow loaded
     * @throws GraphException
     * @throws GPELException
     * @throws ComponentException
     */
    public synchronized Workflow load(URI id, WorkflowType workflowType)
            throws GraphException, GPELException, ComponentException {
        logger.entering(new Object[] { id, workflowType });

        if (!isConnected()) {
            throw new IllegalStateException(
                    "The BPEL Engine has not configured.");
        }

        Workflow workflow = new Workflow();
        switch (workflowType) {
        case TEMPLATE:
            workflow.setGPELTemplateID(id);
            break;
        case INSTANCE:
            workflow.setGPELInstanceID(id);
            break;
        }

        // This is either a workflow template or a workflow instance.
        GcTemplate workflowTemplate;
        GcInstance workflowInstance = null;
        try {
            switch (workflowType) {
            case TEMPLATE:
                workflowTemplate = this.client.retrieveTemplate(id);
                break;
            case INSTANCE:
                // TODO change this when we start modifying the instance.
                workflowInstance = this.client.retrieveInstance(id);
                GcWebResource templateResource = workflowInstance
                        .getLinkWithRel(GpelConstants.REL_TEMPLATE);
                URI templateID = templateResource.getId();
                workflow.setGPELTemplateID(templateID);
                workflowTemplate = this.client.retrieveTemplate(templateID);
                break;
            default:
                // This doesn't happen
                throw new XBayaRuntimeException();
            }
        } catch (GcResourceNotFoundException e) {
            // The workflow was not found in the engine.
            throw new GPELException(
                    ErrorMessages.GPEL_WORKFLOW_NOT_FOUND_ERROR, e);
        } catch (RuntimeException e) {
            throw new GPELException(ErrorMessages.GPEL_ERROR, e);
        }

        GcProcessResource processResource = null;
        if (workflowInstance != null) {
            processResource = (GcProcessResource) workflowInstance
                    .getLinkWithRel(GpelConstants.REL_PROCESS);
        }
        if (processResource == null) {
            processResource = (GcProcessResource) workflowTemplate
                    .getLinkWithRel(GpelConstants.REL_PROCESS);
        }

        GcXmlWebResource graphResource = null;
        if (workflowInstance != null) {
            graphResource = (GcXmlWebResource) workflowInstance
                    .getLinkWithRel(GPELLinksFilter.REL_XWF);
        }
        if (graphResource == null) {
            graphResource = (GcXmlWebResource) workflowTemplate
                    .getLinkWithRel(GPELLinksFilter.REL_XWF);
        }

        GcBinaryWebResource imageResource = null;
        if (workflowInstance != null) {
            imageResource = (GcBinaryWebResource) workflowInstance
                    .getLinkWithRel(GPELLinksFilter.REL_IMAGE);
        }
        if (imageResource == null) {
            imageResource = (GcBinaryWebResource) workflowTemplate
                    .getLinkWithRel(GPELLinksFilter.REL_IMAGE);
        }

        // Look for wsdl resource.
        GcWsdlResource workflowWSDLresource = null;
        Map<String, GcWsdlResource> wsdlResourceMap = new HashMap<String, GcWsdlResource>();
        List<GcWebResource> links = workflowTemplate.getLinks();
        for (GcWebResource link : links) {
            String rel = link.getRel();
            logger.finest("rel: " + rel);
            if (GpelConstants.REL_WSDL.equals(rel)) {
                GcWsdlResource wsdlResouece = (GcWsdlResource) link;
                String wsdlTitle = wsdlResouece.getTitle();
                logger.finest("wsdlTitle: " + wsdlTitle);
                // Use use title to do matching
                wsdlResourceMap.put(wsdlTitle, wsdlResouece);
            }
        }

        // BPEL process
        GpelProcess process = new GpelProcess(processResource.getXmlContent());
        workflow.setGpelProcess(process);

        // WSDL for the process
        workflowWSDLresource = wsdlResourceMap.remove(PROCESS_WSDL_TYTLE);
        WsdlDefinitions workflowWSDL = new WsdlDefinitions(workflowWSDLresource
                .getXmlContent());
        workflow.setWorkflowWSDL(workflowWSDL);

        // WSDLs for services.
        for (String wsdlID : wsdlResourceMap.keySet()) {
            GcWsdlResource wsdlResource = wsdlResourceMap.get(wsdlID);
            WsdlDefinitions definition = new WsdlDefinitions(wsdlResource
                    .getXmlContent());
            workflow.addWSDL(wsdlID, definition);
        }

        // Graph
        WSGraph graph = WSGraphFactory.createGraph(graphResource
                .getXmlContent());
        workflow.setGraph(graph);

        workflow.bindComponents();

        // Set the name of the workflow.
        // This has to be after parsing the graph so that workflow instance name
        // overwrites the name in the graph.
        String name = null;
        if (workflowInstance != null) {
            name = workflowInstance.getTitle();
            logger.finest("name from the instance: " + name);
        }
        if (name == null) {
            name = workflowTemplate.getTitle();
            logger.finest("name from the template: " + name);
        }
        workflow.setName(name);

        // Image
        if (imageResource != null) {
            byte[] imageBytes = imageResource.getBinaryContent();
            try {
                BufferedImage image = ImageIO.read(new ByteArrayInputStream(
                        imageBytes));
                workflow.setImage(image);
            } catch (IOException e) {
                // Not crucial
                logger.caught(e);
            }
        }

        return workflow;
    }

    /**
     * Returns the List of GcSearchResult.
     * 
     * This method returns the first 100 matches.
     * 
     * @return The List of GcSearchResult.
     * @throws GPELException
     */
    public GcSearchList list() throws GPELException {
        return list(100, WorkflowType.TEMPLATE);
    }

    /**
     * Returns the List of GcSearchResult.
     * 
     * @param maxNum
     *            The maximum number of results
     * @param type
     * 
     * @return The List of GcSearchResult.
     * @throws GPELException
     */
    @SuppressWarnings("boxing")
    public synchronized GcSearchList list(int maxNum, WorkflowType type)
            throws GPELException {
        logger.entering(new Object[] { maxNum, type });
        if (!isConnected()) {
            throw new IllegalStateException(
                    "The BPEL Engine has not configured.");
        }

        try {
            // TODO return the ones only related to the user.
            GcSearchList results;
            switch (type) {
            case INSTANCE:
                // TODO no findInstances now.
                throw new UnsupportedOperationException();
            case TEMPLATE:
                results = this.client.findTemplates(maxNum,
                        GcSearchRequest.SearchType.GPEL_TEMPLATE);
                break;
            default:
                throw new XBayaRuntimeException();
            }

            return results;
        } catch (RuntimeException e) {
            throw new GPELException(e);
        }
    }

    /**
     * @param workflow
     * @param dscURL
     * @return The instance of workflow
     * @throws GPELException
     * @throws ComponentException
     * @throws GraphException
     */
    public GcInstance instantiate(Workflow workflow, URI dscURL)
            throws GPELException, ComponentException, GraphException {
        return instantiate(workflow, dscURL, null);
    }

    /**
     * @param workflow
     *            The workflow to instantiate.
     * @param dscURL
     *            The URL of the DSC.
     * @param name
     *            The name that becomes a part of the workflow instance name.
     * @return The instance of workflow
     * @throws GPELException
     * @throws ComponentException
     * @throws GraphException
     */
    public synchronized GcInstance instantiate(Workflow workflow, URI dscURL,
            String name) throws GPELException, ComponentException,
            GraphException {
        if (!isConnected()) {
            throw new IllegalStateException(
                    "The BPEL Engine has not been configured.");
        }

        URI templateID = workflow.getGPELTemplateID();
        if (templateID == null) {
            throw new IllegalStateException(
                    "The workflow has not been deployed.");
        }

        // declare instanceID here to use in the exception handling.
        URI instanceID = null;
        try {
            GcTemplate template = this.client.retrieveTemplate(templateID);
            GcInstance instance = template.createInstance();

            // Set the instance ID to the workflow so that the monitor doesn't
            // load the same workflow.
            instanceID = instance.getInstanceId();
            logger.info("instanceID: " + instanceID);
            workflow.setGPELInstanceID(instanceID);

            // Change the name
            if (name != null) {
                String title = instance.getTitle();
                title = title + " (" + name + ")";
                logger.finest("new title: " + title);
                instance.setTitle(title);
            }

            Graph graph = workflow.getGraph();
            for (WSNode node : GraphUtil.getNodes(graph, WSNode.class)) {
                String partnerLinkName = BPELScript.createPartnerLinkName(node
                        .getID());
                logger.finest("partnerLinkName: " + partnerLinkName);
                WsdlDefinitions wsdl = node.getComponent().getWSDL();
                logger.finest("WSDL QName: " + WSDLUtil.getWSDLQName(wsdl));

                if (WSDLUtil.isAWSDL(wsdl)) {
                    URI subWorkflowTemplateID = WorkflowComponent
                            .getWorkflowTemplateID(wsdl);
                    if (subWorkflowTemplateID == null) {
                        // It's a service AWSDL. Create a CWSDL that uses DSC.
                        logger.finest("service");
                        wsdl = DSCUtil.convertToCWSDL(WSDLUtil.deepClone(wsdl),
                                dscURL);
                    } else {
                        // It's a workflow WSDL.
                        logger.finest("workflow");
                        // recursively instantiate and start sub-workflows.
                        Workflow subWorkflow = load(subWorkflowTemplateID,
                                WorkflowType.TEMPLATE);
                        GcInstance subInstance = instantiate(subWorkflow,
                                dscURL, name);
                        // XXX Need to start to get a sub-workflow CWSDL.
                        // (Not so clean)
                        wsdl = start(subInstance);
                    }
                }

                GcUsesWsdlResource wsdlResource = new GcUsesWsdlResource(
                        partnerLinkName, wsdl);
                instance.getLinks().add(wsdlResource);

                instance.store();
            }
            return instance;
        } catch (RuntimeException e) {
            String message = "Error while creating a workflow instance. template ID: "
                    + templateID + ", instanceID: " + instanceID;
            throw new GPELException(message, e);
        }
    }

    /**
     * Instantiate a specified workflow.
     * 
     * The workflow must have been dployed.
     * 
     * @param workflow
     * @param wsdlMap
     *            Map<partnerLinkName, CWSDL>
     * @return The workflow instance.
     * @throws GPELException
     * 
     * @Deprecated This one doesn't support hierarchical workflows. Use
     *             instantiate(workflow, dscURL) instead.
     */
    @Deprecated
    public synchronized GcInstance instantiate(Workflow workflow,
            Map<String, WsdlDefinitions> wsdlMap) throws GPELException {
        if (!isConnected()) {
            throw new IllegalStateException(
                    "The BPEL Engine has not configured.");
        }

        URI templateID = workflow.getGPELTemplateID();
        if (templateID == null) {
            throw new IllegalStateException(
                    "The workflow has not been deployed.");
        }

        try {
            GcTemplate template = this.client.retrieveTemplate(templateID);
            GcInstance instance = template.createInstance();

            // Set the instance ID to the workflow so that the monitor doesn't
            // load the same workflow.
            URI instanceID = instance.getInstanceId();
            logger.info("instanceID: " + instanceID);
            workflow.setGPELInstanceID(instanceID);

            for (String partnerLinkName : wsdlMap.keySet()) {
                WsdlDefinitions wsdl = wsdlMap.get(partnerLinkName);
                logger.info("partnerLinkName: " + partnerLinkName);
                logger.info("WSDL QName: " + WSDLUtil.getWSDLQName(wsdl));

                GcUsesWsdlResource wsdlResource = new GcUsesWsdlResource(
                        partnerLinkName, wsdl);
                instance.getLinks().add(wsdlResource);

                instance.store();
            }
            return instance;
        } catch (RuntimeException e) {
            throw new GPELException(e);
        }
    }

    /**
     * Starts the workflow instance.
     * 
     * The AWSDLs in workflow must have been modified to CWSDLs.
     * 
     * @param instance
     * @return The WSDL of the workflow.
     * @throws GPELException
     */
    public synchronized WsdlDefinitions start(final GcInstance instance)
            throws GPELException {
        if (!isConnected()) {
            throw new IllegalStateException(
                    "The BPEL Engine has not configured.");
        }

        URI instanceID = null;
        try {
            instanceID = instance.getInstanceId();
            instance.start();

            // access public WSDL for created workflow (contains actual location
            // as SOAP endpoint
            String workflowPartnerLinkName = BPELScript.WORKFLOW_PARTNER_LINK;
            GcProvidesWsdlResource workflowPublicWsdl = (GcProvidesWsdlResource) instance
                    .getLinkWithTitleAndRel(workflowPartnerLinkName,
                            GcProvidesWsdlResource.REL);

            WsdlDefinitions workflowCWSDL = new WsdlDefinitions(
                    workflowPublicWsdl.getXmlContent());
            return workflowCWSDL;
        } catch (RuntimeException e) {
            String message = "Error while starting a workflow instance. instance ID: "
                    + instanceID;
            throw new GPELException(message, e);
        }
    }

    /**
     * @return True if the connection is secure; false otherwise.
     */
    public boolean isSecure() {
        return SecurityUtil.isSecureService(this.engineURL);
    }

    private synchronized void connect() throws GPELException {
        if (this.engineURL == null) {
            if (this.client != null) {
                this.client = null;
                sendSafeEvent(new Event(Event.Type.GPEL_ENGINE_DISCONNECTED));
            }
            return;
        }

        logger.finest("Connecting a GPEL Engine at " + this.engineURL);
        try {
            Transport transport;
            if (isSecure()) {
                if (this.gpelUserX509Credential == null) {
                    logger.finest("Using ssl without any credential.");
                    this.gpelUserX509Credential = new GpelUserX509Credential(
                            null, XBayaSecurity.getTrustedCertificates());
                }
                transport = new Transport(this.gpelUserX509Credential);
            } else {
                // This one is phasing out.
                GpelUserCredentials credentials = new GpelUserCredentials(
                        "user", "password");
                transport = new Transport(credentials);
            }
            this.client = new GpelClient(this.engineURL, transport);
            this.client.setFilter(this.linksFilter);
            sendSafeEvent(new Event(Event.Type.GPEL_ENGINE_CONNECTED));
        } catch (RuntimeException e) {
            this.client = null;
            sendSafeEvent(new Event(Event.Type.GPEL_ENGINE_DISCONNECTED));
            throw new GPELException(ErrorMessages.GPEL_CONNECTION_ERROR, e);
        }
    }

    /**
     * Checks if the client is connected to the BPEL engine.
     * 
     * @return true if the client is connected to the BPEL engine; false
     *         otherwise.
     */
    private synchronized boolean isConnected() {
        return (this.client != null);
    }
}

/*
 * Indiana University Extreme! Lab Software License, Version 1.2
 * 
 * Copyright (c) 2005-2007 The Trustees of Indiana University. All rights
 * reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 * 
 * 2) All redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 * 
 * 3) Any documentation included with all redistributions must include the
 * following acknowledgement:
 * 
 * "This product includes software developed by the Indiana University Extreme!
 * Lab. For further information please visit http://www.extreme.indiana.edu/"
 * 
 * Alternatively, this acknowledgment may appear in the software itself, and
 * wherever such third-party acknowledgments normally appear.
 * 
 * 4) The name "Indiana University" or "Indiana University Extreme! Lab" shall
 * not be used to endorse or promote products derived from this software without
 * prior written permission from Indiana University. For written permission,
 * please contact http://www.extreme.indiana.edu/.
 * 
 * 5) Products derived from this software may not use "Indiana University" name
 * nor may "Indiana University" appear in their name, without prior written
 * permission of the Indiana University.
 * 
 * Indiana University provides no reassurances that the source code provided
 * does not infringe the patent or any other intellectual property rights of any
 * other entity. Indiana University disclaims any liability to any recipient for
 * claims brought by any other entity based on infringement of intellectual
 * property rights or otherwise.
 * 
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS.
 * INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS",
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION
 * GENERATED USING SOFTWARE.
 */
